require 'studio_game/player'
require 'studio_game/treasure_trove'
require 'studio_game/game'

module StudioGame
  describe Player do
    before do
      @player = Player.new("bruno", 150)
      $stdout = StringIO.new
    end

    it "has a capitalized name" do
      expect(@player.name).to eq(@player.name.capitalize)
    end

    it "has an inital health" do
      expect(@player.health).to_not be_nil
      expect(@player.health).to be > 0
    end

    it "has a string representation" do
      expect(@player.to_s).to eq("I'm #{@player.name} with health = #{@player.health}, points = #{@player.points}, and score = #{@player.score}.")
    end

    it "computes a score as the sum of its health and points" do
      @player.found_treasure(Treasure.new(:hammer, 50))
      @player.found_treasure(Treasure.new(:hammer, 50))

      @player.score.should == 250
    end

    it "increases health by 15 when w00ted" do
      initial_health = @player.health
      @player.w00t
      expect(@player.health).to eq(initial_health + 15)
    end

    it "decreases health by 10 when blammed" do
      initial_health = @player.health
      @player.blam
      expect(@player.health).to eq(initial_health - 10)
    end

    context "with a health greater than 100" do
      before do
        @player = Player.new("bruno", 150)
      end

      it "is strong" do
        expect(@player).to be_strong
      end
    end

    context "with an initial health of 100" do
      before do
        @player = Player.new("bruno", 100)
      end

      it "is wimpy" do
        expect(@player).not_to be_strong
      end
    end

    context "in a collection of players" do
      before do
        @player1 = Player.new("moe", 100)
        @player2 = Player.new("larry", 200)
        @player3 = Player.new("curly", 300)

        @players = [@player1, @player2, @player3]
      end

      it "is sorted by decreasing score" do
        expect(@players.sort).to eq([@player3, @player2, @player1])
      end
    end

    it "computes points as the sum of all treasure points" do
      @player.points.should == 0

      @player.found_treasure(Treasure.new(:hammer, 50))

      @player.points.should == 50

      @player.found_treasure(Treasure.new(:crowbar, 400))

      @player.points.should == 450

      @player.found_treasure(Treasure.new(:hammer, 50))

      @player.points.should == 500
    end

    it "assigns a treasure for points during a player's turn" do
      game = Game.new("Knuckleheads")
      player = Player.new("moe")

      game.add_player(player)

      game.play(1)

      expect(player.points).not_to be_zero
    end

    it "yields each found treasure and its total points" do
      @player.found_treasure(Treasure.new(:skillet, 100))
      @player.found_treasure(Treasure.new(:skillet, 100))
      @player.found_treasure(Treasure.new(:hammer, 50))
      @player.found_treasure(Treasure.new(:bottle, 5))
      @player.found_treasure(Treasure.new(:bottle, 5))
      @player.found_treasure(Treasure.new(:bottle, 5))
      @player.found_treasure(Treasure.new(:bottle, 5))
      @player.found_treasure(Treasure.new(:bottle, 5))

      yielded = []
      @player.each_found_treasure do |treasure|
        yielded << treasure
      end

      yielded.should == [
          Treasure.new(:skillet, 200),
          Treasure.new(:hammer, 50),
          Treasure.new(:bottle, 25)
      ]
    end
  end
end